@@ -42,6 +42,7 @@ from .typing import (
     RoborockDeviceProp,
     RoborockCommand,
     RoborockDockSummary,
+    RoborockDeviceInfo,
 )
 
 _LOGGER = logging.getLogger(__name__)
@@ -92,8 +93,8 @@ class PreparedRequest:
 
 class RoborockClient:
 
-    def __init__(self, endpoint: str, device_localkey: dict[str, str]) -> None:
-        self.device_localkey = device_localkey
+    def __init__(self, endpoint: str, devices_info: dict[str, RoborockDeviceInfo]) -> None:
+        self.devices_info = devices_info
         self._seq = 1
         self._random = 4711
         self._id_counter = 10000
@@ -103,21 +104,28 @@ class RoborockClient:
         self._waiting_queue: dict[int, RoborockQueue] = {}
         self._status_listeners: list[Callable[[str, str], None]] = []
 
+    def add_status_listener(self, callback: Callable[[str, str], None]):
+        self._status_listeners.append(callback)
+
+    async def async_disconnect(self) -> Any:
+        raise NotImplementedError
+
     def _decode_msg(self, msg: bytes, local_key: str) -> dict[str, Any]:
         if msg[4:7] == "1.0".encode():
             msg = msg[4:]
         elif msg[0:3] != "1.0".encode():
             raise RoborockException(f"Unknown protocol version {msg[0:3]}")
         if len(msg) == 17:
-            [version, _seq, _random, timestamp, protocol] = struct.unpack(
+            [version, request_id, _random, timestamp, protocol] = struct.unpack(
                 "!3sIIIH", msg[0:17]
             )
             return {
                 "version": version,
+                "request_id": request_id,
                 "timestamp": timestamp,
                 "protocol": protocol,
             }
-        [version, _seq, _random, timestamp, protocol, payload_len] = struct.unpack(
+        [version, request_id, _random, timestamp, protocol, payload_len] = struct.unpack(
             "!3sIIIHH", msg[0:19]
         )
         extra_len = len(msg) - 23 - payload_len
@@ -132,20 +140,21 @@ class RoborockClient:
         decrypted_payload = unpad(decipher.decrypt(payload), AES.block_size) if payload else extra
         return {
             "version": version,
+            "request_id": request_id,
             "timestamp": timestamp,
             "protocol": protocol,
             "payload": decrypted_payload
         }
 
-    def _encode_msg(self, device_id, protocol, timestamp, payload, prefix=None) -> bytes:
-        local_key = self.device_localkey[device_id]
+    def _encode_msg(self, device_id, request_id, protocol, timestamp, payload, prefix=None) -> bytes:
+        local_key = self.devices_info[device_id].local_key
         aes_key = md5bin(encode_timestamp(timestamp) + local_key + self._salt)
         cipher = AES.new(aes_key, AES.MODE_ECB)
         encrypted = cipher.encrypt(pad(payload, AES.block_size))
         encrypted_len = len(encrypted)
         values = [
             "1.0".encode(),
-            self._seq,
+            request_id,
             self._random,
             timestamp,
             protocol,
@@ -164,7 +173,7 @@ class RoborockClient:
 
     async def on_message(self, device_id, msg) -> bool:
         try:
-            data = self._decode_msg(msg, self.device_localkey[device_id])
+            data = self._decode_msg(msg, self.devices_info[device_id].local_key)
             protocol = data.get("protocol")
             if protocol == 102 or protocol == 4:
                 payload = json.loads(data.get("payload").decode())
