@@ -3,14 +3,14 @@ from __future__ import annotations
 import asyncio
 import logging
 import socket
-from asyncio import Lock
-from typing import Callable, Coroutine
+from asyncio import Lock, Future
+from typing import Callable, Coroutine, Any
 
 import async_timeout
 
 from roborock.api import RoborockClient, SPECIAL_COMMANDS
 from roborock.exceptions import RoborockTimeout, CommandVacuumError
-from roborock.typing import RoborockCommand
+from roborock.typing import RoborockCommand, RoborockLocalDeviceInfo
 from roborock.util import get_running_loop_or_create_one
 
 secured_prefix = 199
@@ -21,13 +21,26 @@ set_prefix = 151
 _LOGGER = logging.getLogger(__name__)
 
 
+class RoborockProtocol(asyncio.DatagramProtocol):
+
+    def __init__(self, fut: Future):
+        self.fut = fut
+
+    def datagram_received(self, data, addr):
+        self.fut.set_result((data, addr))
+
+    def error_received(self, exc):
+        self.fut.set_exception(exc)
+
+
 class RoborockLocalClient(RoborockClient):
 
-    def __init__(self, ip: str, device_localkey: dict[str, str]):
-        super().__init__("abc", device_localkey)
+    def __init__(self, devices_info: dict[str, RoborockLocalDeviceInfo]):
+        super().__init__("abc", devices_info)
+        self.loop = get_running_loop_or_create_one()
         self.device_listener: dict[str, RoborockSocketListener] = {
-            device_id: RoborockSocketListener(ip, device_id, self.on_message)
-            for device_id in device_localkey
+            device_id: RoborockSocketListener(device_info.ip, device_id, self.on_message)
+            for device_id, device_info in devices_info.items()
         }
         self._mutex = Lock()
 
@@ -37,6 +50,9 @@ class RoborockLocalClient(RoborockClient):
             for listener in self.device_listener.values()
         ])
 
+    async def async_disconnect(self) -> Any:
+        await asyncio.gather(*[listener.disconnect() for listener in self.device_listener.values()])
+
     async def send_command(
             self, device_id: str, method: RoborockCommand, params: list = None
     ):
@@ -46,7 +62,7 @@ class RoborockLocalClient(RoborockClient):
             _LOGGER.debug(f"id={request_id} Requesting method {method} with {params}")
             prefix = secured_prefix if method in SPECIAL_COMMANDS else get_prefix
             protocol = 4
-            msg = self._encode_msg(device_id, protocol, timestamp, payload, prefix)
+            msg = self._encode_msg(device_id, request_id, protocol, timestamp, payload, prefix)
             # Send the command to the Roborock device
             listener = self.device_listener.get(device_id)
             await listener.send_message(msg)
@@ -87,17 +103,23 @@ class RoborockSocketListener:
                 except Exception as e:
                     _LOGGER.exception(e)
             except BrokenPipeError:
-                self.socket.close()
+                await self.disconnect()
 
     async def connect(self):
         async with self._mutex:
             if not self.is_connected or self.socket.is_closed:
+                self.socket = RoborockSocket(socket.AF_INET, socket.SOCK_STREAM)
+                self.socket.setblocking(False)
                 async with async_timeout.timeout(self.timeout):
                     _LOGGER.info(f"Connecting to {self.ip}")
                     await self.loop.sock_connect(self.socket, (self.ip, 58867))
                     self.is_connected = True
                 self.loop.create_task(self._main_coro())
 
+    async def disconnect(self):
+        self.socket.close()
+        self.is_connected = False
+
     async def send_message(self, data: bytes):
         response = {}
         await self.connect()
@@ -110,5 +132,5 @@ class RoborockSocketListener:
                 f"Timeout after {self.timeout} seconds waiting for response"
             ) from None
         except BrokenPipeError:
-            self.socket.close()
+            await self.disconnect()
         return response
